﻿--
-- New Holland TS115 Functions
-- 
-- author  Mythos
-- date  20/12/2016
-- www.ARM-team.gr
-- Copyright (C) ARM Team, All Rights Reserved.

TS115 = {};

function TS115.prerequisitesPresent(specializations)
    return true;
end;

--________________________________________________________________________________________________________________________________________________________
function TS115:load(savegame)
	
	-- Functions
	self.MPCode = SpecializationUtil.callSpecializationsFunction("MPCode");
	self.playerCallback = SpecializationUtil.callSpecializationsFunction("playerCallback");
	
	-- Indexes
	self.FF1 = Utils.indexToObject(self.components,getXMLString(self.xmlFile,"vehicle.FFEN1#index"));
	self.FF2 = Utils.indexToObject(self.components,getXMLString(self.xmlFile,"vehicle.FFEN2#index"));
	self.BF = Utils.indexToObject(self.components,getXMLString(self.xmlFile,"vehicle.BFEN#index"));
	
	-- Triggers
	self.TS115Triggers = {};
	local i=0;
	while true do
		local key = string.format("vehicle.TS115Triggers.TS115Trigger(%d)", i);
		local trigger = Utils.indexToObject(self.components,getXMLString(self.xmlFile, key.. "#index"));
		if trigger == nil then
			break;
		end;
		self.TS115Triggers[i+1] = {};
		self.TS115Triggers[i+1].trigger = trigger;
		self.TS115Triggers[i+1].enable = false;
		
		addTrigger(trigger, "playerCallback", self);				
		i = i + 1;
	end;
	
	-- Booleans
	self.FFM = true;
	self.BFM = true;

end;

--________________________________________________________________________________________________________________________________________________________
function TS115:delete()
	for i,j in pairs(self.TS115Triggers) do
		if j.trigger ~= nil then
			removeTrigger(j.trigger);
		end;
	end;
end;

--________________________________________________________________________________________________________________________________________________________
function TS115:mouseEvent(posX, posY, isDown, isUp, button)
end;

--________________________________________________________________________________________________________________________________________________________
function TS115:keyEvent(unicode, sym, modifier, isDown)
end;

--________________________________________________________________________________________________________________________________________________________
function TS115:update(dt)
	
	local enable = false;
	local id = 0;
	for i,j in pairs(self.TS115Triggers) do
		if j.enable then
			id = i;
			enable = true;
			break;
		end;
	end;
	
	if enable then
		if id==1 then
			g_currentMission:addHelpButtonText(string.format(g_i18n:getText("FEN_MODE"), self.typeDesc), InputBinding.IMPLEMENT_EXTRA2);
			if InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA2) then
				self:MPCode(1, not self.FFM); -- Front Fenders
			end;
		elseif id==2 then
			g_currentMission:addHelpButtonText(string.format(g_i18n:getText("FEN_MODE"), self.typeDesc), InputBinding.IMPLEMENT_EXTRA2);
			if InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA2) then
				self:MPCode(2, not self.BFM); -- Back Fenders
			end;
		end;
	end;
	
end;

--________________________________________________________________________________________________________________________________________________________
function TS115:updateTick(dt)
end;

--________________________________________________________________________________________________________________________________________________________
function TS115:MPCode(mode, state, a1, noEventSend)
	TS115Event.sendEvent(self, mode, state, a1, noEventSend);

--______________________________________________________________________________
	if mode == 1 then
--_____________________________________ FRONT FENDER MODE ______________________________
		if state then
			setVisibility(self.FF1, true);
			setVisibility(self.FF2, true);
			self.FFM = true;
		else
			setVisibility(self.FF1, false);
			setVisibility(self.FF2, false);
			self.FFM = false;
		end;
--______________________________________________________________________________
	elseif mode == 2 then
--_____________________________________ FRONT FENDER MODE ______________________________
		if state then
			setVisibility(self.BF, true);
			self.BFM = true;
		else
			setVisibility(self.BF, false);
			self.BFM = false;
		end;
	end;

end;

--________________________________________________________________________________________________________________________________________________________
function TS115:draw()
end;

--________________________________________________________________________________________________________________________________________________________
--________________________________________________________________________________________________________________________________________________________
function TS115:postLoad(savegame)
	if savegame ~= nil and not savegame.resetVehicles then
		self.FFM = Utils.getNoNil(getXMLBool(savegame.xmlFile, savegame.key.."#FFM"),true);
		self.BFM = Utils.getNoNil(getXMLBool(savegame.xmlFile, savegame.key.."#BFM"),true);

		self:MPCode(1, self.FFM);
		self:MPCode(2, self.BFM);
	end;

end;

--________________________________________________________________________________________________________________________________________________________
function TS115:getSaveAttributesAndNodes(nodeIdent)
	local attributes = 'FFM="'..tostring(self.FFM)..
	'" BFM="'..tostring(self.BFM)..'"';

	local nodes = nil;
	
	return attributes,nodes;
end;

--________________________________________________________________________________________________________________________________________________________
function TS115:readStream(streamId, connection)
	self.FFM = streamReadBool(streamId);
	self.BFM = streamReadBool(streamId);
	
	self:MPCode(1, self.FFM, nil, true);
	self:MPCode(2, self.BFM, nil, true);
	
end;

--________________________________________________________________________________________________________________________________________________________
function TS115:writeStream(streamId, connection)
	streamWriteBool(streamId, self.FFM);
	streamWriteBool(streamId, self.BFM);
end;

--________________________________________________________________________________________________________________________________________________________
function TS115:playerCallback(triggerId, otherId, onEnter, onLeave, onStay)
	
	local id = 0;
	for i,j in pairs(self.TS115Triggers) do
		if j.trigger == triggerId then
			id = i;
			break;
		end;
	end;
	
	if id ~= 0 then
		if onEnter and g_currentMission.controlPlayer and g_currentMission.player ~= nil and otherId == g_currentMission.player.rootNode then
			self.TS115Triggers[id].enable = true;
		elseif onLeave then
			self.TS115Triggers[id].enable = false;
		end;
	end
	
end;

--**********************************************************************************************************************************************************************************
TS115Event = {};
TS115Event_mt = Class(TS115Event, Event);

InitEventClass(TS115Event, "TS115Event");

function TS115Event:emptyNew()
    local self = Event:new(TS115Event_mt);
    self.className="TS115Event";
    return self;
end;

function TS115Event:new(vehicle, mode, state, a1)
    local self = TS115Event:emptyNew()
    self.vehicle = vehicle;
	self.mode = mode;
	self.state = state;
	self.a1 = a1;
    return self;
end;

function TS115Event:readStream(streamId, connection)
    local id = streamReadInt32(streamId);
	self.mode = streamReadInt32(streamId);
	self.state = streamReadBool(streamId);
	self.a1 = streamReadInt32(streamId);
    self.vehicle = networkGetObject(id);
    self:run(connection);
end;

function TS115Event:writeStream(streamId, connection)
    streamWriteInt32(streamId, networkGetObjectId(self.vehicle));
	streamWriteInt32(streamId, self.mode);
	streamWriteBool(streamId, self.state);
	streamWriteInt32(streamId, self.a1);
end;

function TS115Event:run(connection)
	self.vehicle:MPCode(self.mode, self.state, self.a1, true);
    if not connection:getIsServer() then
        g_server:broadcastEvent(TS115Event:new(self.vehicle, self.mode, self.state, self.a1), nil, connection, self.vehicle);
    end;
end;

function TS115Event.sendEvent(vehicle, mode, state, a1, noEventSend)
	if noEventSend == nil or noEventSend == false then
		if g_server ~= nil then
			g_server:broadcastEvent(TS115Event:new(vehicle, mode, state, a1), nil, nil, vehicle);
		else
			g_client:getServerConnection():sendEvent(TS115Event:new(vehicle, mode, state, a1));
		end;
	end;
end;